@@ -20,6 +20,11 @@
  * cause server crashs or bad performance.
  */
 
+/*
+ * maximal listener sockets
+ */
+#define LST	32
+
 /*
  * maximal concurrent connections
  */
@@ -42,7 +47,7 @@
 
 /*
  * socket send buffer size (bytes), 0 = keep operating system default
- */ 
+ */
 #define SBS	0
 
 /*
