@@ -181,7 +181,7 @@ http_verify(const char *header, const char *cip, const int sfd, const int hr)
 		free(b);
 		return (0);
 	}
-	
+
 	/* access log 1 client ip, date, request string */
 	time(&tnow);
 	t = localtime(&tnow);
@@ -227,7 +227,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 
 	if (rh->rp_status == 301)
 		x = http_body(http_s_301, "", rh->rp_header, rh->x_chk);
-	if (rh->rp_status == 304) 
+	if (rh->rp_status == 304)
 		x = strdup(rh->rp_header);
 	if (rh->rp_status == 401) {
 		x = http_body(http_s_401, "", rh->rp_header, rh->x_chk);
@@ -340,7 +340,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 		if (rh->x_agt == 0) {
 			c[sfd].plref[hr][0] = '-';
 			c[sfd].plref[hr][1] = '\0';
-		} else 
+		} else
 			strlcpy(c[sfd].plagt[hr], rh->rq_fv_agt, 128);
 
 		strlcpy(c[sfd].pllog[hr], logn, 128);
@@ -662,11 +662,11 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 		if (!x_nph && rh->x_chk) {
 			if (c[sfd].x_ssl) {
 				if ((sp = sys_write_ssl(c[sfd].ssl, http_fv_lch,
-			    	    strlen(http_fv_lch))) != -1)
+				    strlen(http_fv_lch))) != -1)
 					st += sp;
 			} else {
 				if ((sp = sys_write(sfd, http_fv_lch,
-			    	    strlen(http_fv_lch))) != -1)
+				    strlen(http_fv_lch))) != -1)
 					st += sp;
 			}
 		}
@@ -784,7 +784,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 						break;
 				} else {
 					if ((sp = sys_write(sfd, x, strlen(x)))
-					     == -1)
+					    == -1)
 						break;
 				}
 				st += sp;
@@ -853,7 +853,7 @@ http_proc(const char *header, char *body, const int hr, const int blen,
 	if (rh->rp_status == 200 || rh->rp_status == 206) {
 		snprintf(tmp, sizeof(tmp), "%d %d", rh->rp_status, st);
 		strlcat(log_1, tmp, sizeof(log_1));
-	} else { 
+	} else {
 		snprintf(tmp, sizeof(tmp), "%d -", rh->rp_status);
 		strlcat(log_1, tmp, sizeof(log_1));
 	}
@@ -945,7 +945,7 @@ http_cgi_header(char *header_cgi, char *status, const int dsize1,
 {
 	int	i, j, r;
 	char	line[1024], option[1024], tmp[1024], header_cgi_new[1024];
-	
+
 	r = 0;
 
 	memset(header_cgi_new, 0, sizeof(header_cgi_new));
@@ -1023,7 +1023,7 @@ int
 http_body_comp(char *body, const int blen, const int brec, const int hlen)
 {
 	int	r;
-	
+
 	r = 0;
 
 	/* handle the post body termination mess */
@@ -1148,7 +1148,7 @@ http_chunk(const char *block, const int block_size)
 	memcpy(chunk + strlen(hex), block, block_size);
 	memcpy(chunk + strlen(hex) + block_size, "\r\n\0", 3);
 
-	return (chunk); 
+	return (chunk);
 }
 
 /*
@@ -1196,7 +1196,7 @@ http_uridecode(const char *uri)
 {
 	int	i, j, k, found;
 	char	*dst, hex[3];
-	
+
 	found = 0;
 
 	if ((dst = malloc(strlen(uri) + 1)) == NULL)
@@ -1421,7 +1421,7 @@ http_header(const char *header_data, const char *force_status,
 			free(x);
 		}
 	}
-	
+
 	/* is there a query string */
 	if (strcuts(h->rq_query, h->rq_uri, '?', '\0', sizeof(h->rq_query))
 	    != -1) {
@@ -1543,6 +1543,15 @@ http_header(const char *header_data, const char *force_status,
 			/* remove ~ */
 			strlcpy(file_path + 1, h->rq_uri + 2,
 			    sizeof(file_path));
+			/* insert homedirs sub directory if configured */
+			if (config.homedirs_public[0] != '\0') {
+				strcuts(tmp, file_path, '\0', '/', sizeof(tmp));
+				i = strlen(tmp);
+				strlcat(tmp, config.homedirs_public,
+				    sizeof(tmp));
+				strlcat(tmp, file_path + i, sizeof(tmp));
+				strlcpy(file_path, tmp, sizeof(file_path));
+			}
 		}
 	}
 
@@ -1562,7 +1571,7 @@ http_header(const char *header_data, const char *force_status,
 	snprintf(h->rq_filef, sizeof(h->rq_filef), "%s%s", docroot, file_path);
 
 	/* save file and path separated */
-	strcutf(h->rq_files, h->rq_filep, h->rq_filef, sizeof(h->rq_files), 
+	strcutf(h->rq_files, h->rq_filep, h->rq_filef, sizeof(h->rq_files),
 	    sizeof(h->rq_filep));
 
 	/* check file */
@@ -1664,16 +1673,17 @@ http_header(const char *header_data, const char *force_status,
 			    sizeof(h->rp_fv_dat));
 			h->rp_fsize = s.st_size;
 		}
-	}
 
-	/* do we have permissions */
-	if (!(s.st_mode & S_IROTH)) {
-		/* nope */
-		h->x_idx = 0;
-		/* 403 */
-		strlcpy(status, http_s_403, sizeof(status));
-		strlcpy(h->rp_fv_cte, http_fv_cte, sizeof(h->rp_fv_cte));
-		h->rp_status = 403;
+		/* do we have permissions */
+		if (!(s.st_mode & S_IROTH)) {
+			/* nope */
+			h->x_idx = 0;
+			/* 403 */
+			strlcpy(status, http_s_403, sizeof(status));
+			strlcpy(h->rp_fv_cte, http_fv_cte,
+			    sizeof(h->rp_fv_cte));
+			h->rp_status = 403;
+		}
 	}
 
 	/* was the file modified */
@@ -1735,7 +1745,7 @@ http_header(const char *header_data, const char *force_status,
 				/* 401 */
 				strlcpy(status, http_s_401, sizeof(status));
 				strlcpy(h->rp_fv_cte, http_fv_cte,
-				    sizeof(h->rp_fv_cte)); 
+				    sizeof(h->rp_fv_cte));
 				h->x_cgi = 0;
 				h->rp_status = 401;
 #ifdef __OpenBSD__
@@ -1846,6 +1856,6 @@ http_header(const char *header_data, const char *force_status,
 		    http_fv_srv, r == 0 ? tmp : h->rq_fv_hos,
 		    c[sfd].x_ssl ? config.sslportc : config.serverportc);
 	}
-		    
+
 	return (h);
 }
