@@ -38,7 +38,7 @@
 #include <string.h>
 #include <syslog.h>
 #include <time.h>
-#include <unistd.h> 
+#include <unistd.h>
 
 #include "../libmy/str.h"
 #include "../libmy/file.h"
@@ -109,7 +109,7 @@ const char *http_fn_auw		= "WWW-Authenticate:";
 const char *http_fn_ran		= "Range:";
 const char *http_fn_rac		= "Content-Range:";
 const char *http_fn_aen		= "Accept-Encoding:";
-const char *http_fv_srv		= "nostromo 1.9.1";
+const char *http_fv_srv		= "nostromo 1.9.3";
 const char *http_fv_pr0	=	 "HTTP/1.0";
 const char *http_fv_pr1	=	 "HTTP/1.1";
 const char *http_fv_cgi		= "CGI/1.1";
@@ -127,6 +127,7 @@ char	http_path[128];
 
 struct cfg		config;
 struct connection	c[CON];
+struct listener		l[LST + 1];
 
 /*
  * usage()
@@ -382,6 +383,33 @@ send_file(const int sfd, const int hr)
 	}
 }
 
+/*
+ * sdlisten()
+ *	checks all listener sockets for ``sd'' and ``sdtype''
+ * Return:
+ *	0 = no listener socket has matched, 1 = a listener socket has matched
+ */
+int
+sdlisten(int sd, int sdtype)
+{
+	int i;
+
+	for (i = 0; l[i].sd; i++) {
+		if (sd == l[i].sd)
+			break;
+	}
+
+	if (l[i].sd == 0)
+		/* not a listener socket */
+		return (0);
+
+	if ((l[i].sdtype & sdtype) == 0) 
+		/* not the listener type we want */
+		return (0);
+
+	return (1);
+}
+
 /*
  * main()
  *	nostromo webserver
@@ -391,7 +419,6 @@ send_file(const int sfd, const int hr)
 int
 main(int argc, char *argv[])
 {
-	int			sdlis1, sdlis2, sdlis3, sdlis4;
 	int			sdmax, sdnew, sdnow, sdlim;
 	int			i, j, k, r, s, ch, rd, rt, wr, post, size;
 	int			fd, erro, opt_c, opt_r, opt_4, opt_6;
@@ -411,6 +438,8 @@ main(int argc, char *argv[])
 	struct sockaddr_in	sa, ca;
 	struct sockaddr_in6	sa6, ca6;
 	struct in6_addr		in6addr_any = IN6ADDR_ANY_INIT;
+	struct in_addr		ina;
+	struct in6_addr		ina6;
 	struct timeval		tv;
 	socklen_t		ssize;
 	fd_set			master_r, master_w, set_r, set_w;
@@ -419,7 +448,6 @@ main(int argc, char *argv[])
 	FILE			*file;
 
 	i = r = s = rd = sdmax = sdnow = 0;
-	sdlis1 = sdlis2 = sdlis3 = sdlis4 = -1;
 
 	/*
 	 * command line arguments
@@ -461,7 +489,7 @@ main(int argc, char *argv[])
 		strlcpy(config.file, configfile, sizeof(config.file));
 	/* check if configuration file exists */
 	if ((fd = open(config.file, O_RDONLY)) == -1)
-		err(1, config.file);
+		err(1, "%s", config.file);
 	else
 		close(fd);
 
@@ -490,6 +518,11 @@ main(int argc, char *argv[])
 		strcuts(config.servername, config.servername, '\0', ':',
 		    sizeof(config.servername));
 	}
+	if (fparse(config.serverlisten, "serverlisten", config.file,
+	    sizeof(config.serverlisten)) == -1) {
+		fprintf(stderr, "Missing <serverlisten> in config file.\n");
+		r = -1;
+	}
 	if (fparse(config.serveradmin, "serveradmin", config.file,
 	    sizeof(config.serveradmin)) == -1) {
 		fprintf(stderr, "Missing <serveradmin> in config file.\n");
@@ -579,6 +612,16 @@ main(int argc, char *argv[])
 	if (fparse(config.homedirs, "homedirs", config.file,
 	    sizeof(config.homedirs)) == -1)
 		config.homedirs[0] = '\0';
+	if (fparse(config.homedirs_public, "homedirs_public", config.file,
+	    sizeof(config.homedirs_public)) == -1) {
+		config.homedirs_public[0] = '\0';
+	} else {
+		if (config.homedirs_public[0] != '/') {
+			strlcpy(tmp, config.homedirs_public, sizeof(tmp));
+			snprintf(config.homedirs_public,
+			    sizeof(config.homedirs_public), "/%s", tmp);
+		}
+	}
 
 	/* we need all mandatory parameters otherwise no start */
 	if (r == -1)
@@ -604,7 +647,7 @@ main(int argc, char *argv[])
 	stat(config.servermimes, &st);
 	mimes_size = st.st_size;
 	if ((fd = open(config.servermimes, O_RDONLY, 0)) == -1)
-		err(1, config.servermimes);
+		err(1, "%s", config.servermimes);
 	else {
 		if ((mimes = mmap(NULL, mimes_size, PROT_READ,
 		    MAP_FILE|MAP_SHARED, fd, 0)) == MAP_FAILED)
@@ -692,162 +735,300 @@ main(int argc, char *argv[])
 	FD_ZERO(&set_r);
 	FD_ZERO(&set_w);
 
+	i = 0;
 	/*
 	 * IPv4
 	 */
 	if (opt_4) {
-		/* socket */
-		if ((sdlis1 = socket(AF_INET, SOCK_STREAM, 0)) == -1)
-			err(1, "socket");
+		if ((b = strdup(config.serverlisten)) == NULL)
+			err(1, "strdup");
 
-		/* set socket non-blocking */
-		if (fcntl(sdlis1, F_SETFL, fcntl(sdlis1, F_GETFL) |
-		    O_NONBLOCK) == -1)
-			err(1, "fcntl");
-		/* allow rapid reuse of the address */
-		size = 1;
-		if (setsockopt(sdlis1, SOL_SOCKET, SO_REUSEADDR, (char *) &size,
-		    sizeof(size)) == -1)
-			err(1, "setsockopt");
+		r = 0;
+		for (h = strtok(b, " "); h; h = strtok(NULL, " ")) {
+			/* check for free space in listener socket array */
+			if (i == LST)
+				break;
 
-		/* bind */
-		sa.sin_family = AF_INET;
-		sa.sin_port = htons(config.serverport);
-		sa.sin_addr.s_addr = INADDR_ANY;
-		memset(&sa.sin_zero, 0, sizeof(sa.sin_zero));
-		if (bind(sdlis1, (struct sockaddr *)&sa, sizeof(sa)) == -1)
-			err(1, "bind");
+			/* if no IPv4 address nor wildcard, skip it */
+			if (strchr(h, '.') == NULL && *h != '*')
+				continue;
 
-		/* listen */
-		if (listen(sdlis1, CON) == -1)
-			err(1, "listen");
+			/* socket */
+			if ((l[i].sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
+				err(1, "socket");
 
-		/* add listener socket to master set */
-		FD_SET(sdlis1, &master_r);
+			/* set socket non-blocking */
+			if (fcntl(l[i].sd, F_SETFL,
+			    fcntl(l[i].sd, F_GETFL) | O_NONBLOCK) == -1)
+				err(1, "fcntl");
+			/* allow rapid reuse of the address */
+			size = 1;
+			if (setsockopt(l[i].sd, SOL_SOCKET, SO_REUSEADDR,
+			    (char *)&size, sizeof(size)) == -1)
+				err(1, "setsockopt");
 
-		/* keep track of the biggest socket descriptor */
-		sdmax = sdlis1;
+			/* bind */
+			sa.sin_family = AF_INET;
+			sa.sin_port = htons(config.serverport);
+			if (*h == '*') {
+				sa.sin_addr.s_addr = INADDR_ANY;
+				r = 1;
+			} else {
+				bzero(&ina, sizeof(struct in_addr));
+				if (inet_pton(AF_INET, h, &ina) != 1)
+					err(1, "inet_pton");
+				sa.sin_addr.s_addr = ina.s_addr;
+			}
+			memset(&sa.sin_zero, 0, sizeof(sa.sin_zero));
+			if (bind(l[i].sd, (struct sockaddr *)&sa,
+			    sizeof(sa)) == -1)
+				err(1, "bind");
+
+			/* listen */
+			if (listen(l[i].sd, CON) == -1)
+				err(1, "listen");
+
+			/* add listener socket to master set */
+			FD_SET(l[i].sd, &master_r);
+
+			/* keep track of the biggest socket descriptor */
+			sdmax = l[i].sd;
+
+			l[i].sdtype = IPALL | IP4;
+			i++;
+
+			/* we use INADDR_ANY, done */
+			if (r)
+				break;
+		}
+
+		free(b);
 	}
 
 	/*
 	 * IPv4 SSL
 	 */
 	if (opt_4 && config.ssl) {
-		/* socket */
-		if ((sdlis2 = socket(AF_INET, SOCK_STREAM, 0)) == -1)
-			err(1, "socket");
+		if ((b = strdup(config.serverlisten)) == NULL)
+			err(1, "strdup");
 
-		/* set socket non-blocking */
-		if (fcntl(sdlis2, F_SETFL, fcntl(sdlis2, F_GETFL) |
-		    O_NONBLOCK) == -1)
-			err(1, "fcntl");
-		/* allow rapid reuse of the address */
-		size = 1;
-		if (setsockopt(sdlis2, SOL_SOCKET, SO_REUSEADDR, (char *) &size,
-		    sizeof(size)) == -1)
-			err(1, "setsockopt");
+		r = 0;
+		for (h = strtok(b, " "); h; h = strtok(NULL, " ")) {
+			/* check for free space in listener socket array */
+			if (i == LST)
+				break;
 
-		/* bind */
-		sa.sin_family = AF_INET;
-		sa.sin_port = htons(config.sslport);
-		sa.sin_addr.s_addr = INADDR_ANY;
-		memset(sa.sin_zero, 0, sizeof(sa.sin_zero));
-		if (bind(sdlis2, (struct sockaddr *)&sa, sizeof(sa)) == -1)
-			err(1, "bind");
+			/* if no IPv4 address nor wildcard, skip it */
+			if (strchr(h, '.') == NULL && *h != '*')
+				continue;
 
-		/* listen */
-		if (listen(sdlis2, CON) == -1)
-			err(1, "listen");
+			/* socket */
+			if ((l[i].sd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
+				err(1, "socket");
 
-		/* add ssl listener socket to master set */
-		FD_SET(sdlis2, &master_r);
+			/* set socket non-blocking */
+			if (fcntl(l[i].sd, F_SETFL,
+			    fcntl(l[i].sd, F_GETFL) | O_NONBLOCK) == -1)
+				err(1, "fcntl");
+			/* allow rapid reuse of the address */
+			size = 1;
+			if (setsockopt(l[i].sd, SOL_SOCKET, SO_REUSEADDR,
+			    (char *)&size, sizeof(size)) == -1)
+				err(1, "setsockopt");
 
-		/* keep track of the biggest socket descriptor */
-		sdmax = sdlis2;
+			/* bind */
+			sa.sin_family = AF_INET;
+			sa.sin_port = htons(config.sslport);
+			if (*h == '*') {
+				sa.sin_addr.s_addr = INADDR_ANY;
+				r = 1;
+			} else {
+				bzero(&ina, sizeof(struct in_addr));
+				if (inet_pton(AF_INET, h, &ina) != 1)
+					err(1, "inet_pton");
+				sa.sin_addr.s_addr = ina.s_addr;
+			}
+			memset(sa.sin_zero, 0, sizeof(sa.sin_zero));
+			if (bind(l[i].sd, (struct sockaddr *)&sa,
+			    sizeof(sa)) == -1)
+				err(1, "bind");
+
+			/* listen */
+			if (listen(l[i].sd, CON) == -1)
+				err(1, "listen");
+
+			/* add ssl listener socket to master set */
+			FD_SET(l[i].sd, &master_r);
+
+			/* keep track of the biggest socket descriptor */
+			sdmax = l[i].sd;
+
+			l[i].sdtype = IPALL | IP4SSL;
+			i++;
+
+			/* we use INADDR_ANY, done */
+			if (r)
+				break;
+		}
+
+		free(b);
 	}
 
 	/*
 	 * IPv6
 	 */
 	if (opt_6) {
-		/* socket */
-		if ((sdlis3 = socket(AF_INET6, SOCK_STREAM, 0)) == -1)
-			err(1, "socket");
+		if ((b = strdup(config.serverlisten)) == NULL)
+			err(1, "strdup");
 
-		/* set socket non-blocking */
-		if (fcntl(sdlis3, F_SETFL, fcntl(sdlis3, F_GETFL) |
-		    O_NONBLOCK) == -1)
-			err(1, "fcntl");
-		/* allow rapid reuse of the address */
-		size = 1;
-		if (setsockopt(sdlis3, SOL_SOCKET, SO_REUSEADDR, (char *) &size,
-		    sizeof(size)) == -1)
-			err(1, "setsockopt");
+		r = 0;
+		for (h = strtok(b, " "); h; h = strtok(NULL, " ")) {
+			/* check for free space in listener socket array */
+			if (i == LST)
+				break;
 
-		/* bind */
-		#ifndef __linux__
-		sa6.sin6_len = sizeof(sa6);
-		#endif
-		sa6.sin6_family = AF_INET6;
-		sa6.sin6_port = htons(config.serverport);
-		sa6.sin6_flowinfo = 0;
-		sa6.sin6_scope_id = 0;
-		memcpy(sa6.sin6_addr.s6_addr, &in6addr_any,
-		    sizeof(in6addr_any));
-		if (bind(sdlis3, (struct sockaddr *)&sa6, sizeof(sa6)) == -1)
-			err(1, "bind");
+			/* if no IPv4 address nor wildcard, skip it */
+			if (strchr(h, ':') == NULL && *h != '*')
+				continue;
 
-		/* listen */
-		if (listen(sdlis3, CON) == -1)
-			err(1, "listen");
+			/* socket */
+			if ((l[i].sd = socket(AF_INET6, SOCK_STREAM, 0)) == -1)
+				err(1, "socket");
 
-		/* add ipv6 listener to master set */
-		FD_SET(sdlis3, &master_r);
+			/* set socket non-blocking */
+			if (fcntl(l[i].sd, F_SETFL,
+			    fcntl(l[i].sd, F_GETFL) | O_NONBLOCK) == -1)
+				err(1, "fcntl");
+			/* allow rapid reuse of the address */
+			size = 1;
+			if (setsockopt(l[i].sd, SOL_SOCKET, SO_REUSEADDR,
+			    (char *)&size, sizeof(size)) == -1)
+				err(1, "setsockopt");
 
-		/* keep track of the biggest socket descriptor */
-		sdmax = sdlis3;
+			/* bind */
+			#ifndef __linux__
+			sa6.sin6_len = sizeof(sa6);
+			#endif
+			sa6.sin6_family = AF_INET6;
+			sa6.sin6_port = htons(config.serverport);
+			sa6.sin6_flowinfo = 0;
+			sa6.sin6_scope_id = 0;
+			if (*h == '*') {
+				memcpy(sa6.sin6_addr.s6_addr, &in6addr_any,
+				    sizeof(in6addr_any));
+				r = 1;
+			} else {
+				//bzero(&ina6, sizeof(struct in6_addr));
+				if (inet_pton(AF_INET6, h, &ina6.s6_addr) != 1)
+					err(1, "inet_pton");
+				memcpy(sa6.sin6_addr.s6_addr, &ina6.s6_addr,
+				    sizeof(struct in6_addr));
+			}
+			if (bind(l[i].sd, (struct sockaddr *)&sa6,
+			    sizeof(sa6)) == -1)
+				err(1, "bind");
+
+			/* listen */
+			if (listen(l[i].sd, CON) == -1)
+				err(1, "listen");
+
+			/* add ipv6 listener to master set */
+			FD_SET(l[i].sd, &master_r);
+
+			/* keep track of the biggest socket descriptor */
+			sdmax = l[i].sd;
+
+			l[i].sdtype = IPALL | IP6;
+			i++;
+
+			/* we use INADDR_ANY, done */
+			if (r)
+				break;
+		}
+
+		free(b);
 	}
 
 	/*
 	 * IPv6 SSL
 	 */
 	if (opt_6 && config.ssl) {
-		/* socket */
-		if ((sdlis4 = socket(AF_INET6, SOCK_STREAM, 0)) == -1)
-			err(1, "socket");
+		if ((b = strdup(config.serverlisten)) == NULL)
+			err(1, "strdup");
 
-		/* set socket non-blocking */
-		if (fcntl(sdlis4, F_SETFL, fcntl(sdlis4, F_GETFL) |
-		    O_NONBLOCK) == -1)
-			err(1, "fcntl");
-		/* allow rapid reuse of the address */
-		size = 1;
-		if (setsockopt(sdlis4, SOL_SOCKET, SO_REUSEADDR, (char *) &size,
-		    sizeof(size)) == -1)
-			err(1, "setsockopt");
+		r = 0;
+		for (h = strtok(b, " "); h; h = strtok(NULL, " ")) {
+			/* check for free space in listener socket array */
+			if (i == LST)
+				break;
 
-		/* bind */
-		#ifndef __linux__
-		sa6.sin6_len = sizeof(sa6);
-		#endif
-		sa6.sin6_family = AF_INET6;
-		sa6.sin6_port = htons(config.sslport);
-		sa6.sin6_flowinfo = 0;
-		sa6.sin6_scope_id = 0;
-		memcpy(sa6.sin6_addr.s6_addr, &in6addr_any,
-		    sizeof(in6addr_any));
-		if (bind(sdlis4, (struct sockaddr *)&sa6, sizeof(sa6)) == -1)
-			err(1, "bind");
+			/* if no IPv4 address nor wildcard, skip it */
+			if (strchr(h, ':') == NULL && *h != '*')
+				continue;
 
-		/* listen */
-		if (listen(sdlis4, CON) == -1)
-			err(1, "listen");
+			/* socket */
+			if ((l[i].sd = socket(AF_INET6, SOCK_STREAM, 0)) == -1)
+				err(1, "socket");
 
-		/* add ipv6 listener to master set */
-		FD_SET(sdlis4, &master_r);
+			/* set socket non-blocking */
+			if (fcntl(l[i].sd, F_SETFL,
+			    fcntl(l[i].sd, F_GETFL) | O_NONBLOCK) == -1)
+				err(1, "fcntl");
+			/* allow rapid reuse of the address */
+			size = 1;
+			if (setsockopt(l[i].sd, SOL_SOCKET, SO_REUSEADDR,
+			    (char *)&size, sizeof(size)) == -1)
+				err(1, "setsockopt");
 
-		/* keep track of the biggest socket descriptor */
-		sdmax = sdlis4;
+			/* bind */
+			#ifndef __linux__
+			sa6.sin6_len = sizeof(sa6);
+			#endif
+			sa6.sin6_family = AF_INET6;
+			sa6.sin6_port = htons(config.sslport);
+			sa6.sin6_flowinfo = 0;
+			sa6.sin6_scope_id = 0;
+			if (*h == '*') {
+				memcpy(sa6.sin6_addr.s6_addr, &in6addr_any,
+				    sizeof(in6addr_any));
+			} else {
+				//bzero(&ina6, sizeof(struct in6_addr));
+				if (inet_pton(AF_INET6, h, &ina6.s6_addr) != 1)
+					err(1, "inet_pton");
+				memcpy(sa6.sin6_addr.s6_addr, &ina6.s6_addr,
+				    sizeof(struct in6_addr));
+			}
+			if (bind(l[i].sd, (struct sockaddr *)&sa6,
+			    sizeof(sa6)) == -1)
+				err(1, "bind");
+
+			/* listen */
+			if (listen(l[i].sd, CON) == -1)
+				err(1, "listen");
+
+			/* add ipv6 listener to master set */
+			FD_SET(l[i].sd, &master_r);
+
+			/* keep track of the biggest socket descriptor */
+			sdmax = l[i].sd;
+
+			l[i].sdtype = IPALL | IP6SSL;
+			i++;
+
+			/* we use INADDR_ANY, done */
+			if (r)
+				break;
+		}
+
+		free(b);
+	}
+
+	/*
+	 * check if we could bind at least to one interface
+	 */
+	if (i == 0) {
+		fprintf(stderr, "Couldn't bind to any interfaces, aborting.\n");
+		exit(1);
 	}
 
 	/*
@@ -898,10 +1079,10 @@ main(int argc, char *argv[])
 	dup(0);
 
 #ifdef __OpenBSD__
-        /*
-         * basic authentication via bsd authentication framework
-         */
-        if (config.bsdauth) {
+	/*
+	 * basic authentication via bsd authentication framework
+	 */
+	if (config.bsdauth) {
 		pipe(fdbsdauth);
 		pid = fork();
 		if (pid == -1)
@@ -1008,9 +1189,7 @@ main(int argc, char *argv[])
 			/*
 			 * Unused connection
 			 */
-			if (c[sdnow].to == 0 && sdnow != sdlis1 &&
-			    sdnow != sdlis2 && sdnow != sdlis3 &&
-			    sdnow != sdlis4)
+			if (c[sdnow].to == 0 && sdlisten(sdnow, IPALL) == 0)
 				continue;
 
 			/* check connection read and write state */
@@ -1021,8 +1200,7 @@ main(int argc, char *argv[])
 			 * existing connection idles, check for timeout
 			 */
 			if (rd == 0 && wr == 0) {
-				if (sdnow != sdlis1 && sdnow != sdlis2 &&
-				    sdnow != sdlis3 && sdnow != sdlis4 &&
+				if (sdlisten(sdnow, IPALL) == 0 &&
 				    c[sdnow].pfdo == 0 &&
 				    (time(&tnow) - c[sdnow].to) > TON) {
 					sys_log(debug,
@@ -1035,10 +1213,9 @@ main(int argc, char *argv[])
 			/*
 			 * new connection
 			 */
-			if (rd > 0 && (sdnow == sdlis1 || sdnow == sdlis2 ||
-			    sdnow == sdlis3 || sdnow == sdlis4)) {
+			if (rd > 0 && sdlisten(sdnow, IPALL)) {
 				/* accept */
-				if (sdnow == sdlis3 || sdnow == sdlis4) {
+				if (sdlisten(sdnow, IP6 | IP6SSL)) {
 					ssize = sizeof(ca6);
 					if ((sdnew = accept(sdnow,
 					    (struct sockaddr *)&ca6, &ssize)) ==
@@ -1077,14 +1254,14 @@ main(int argc, char *argv[])
 				c[sdnew].x_sta = 0;
 
 				/* SSL ready */
-				if (sdnow == sdlis2 || sdnow == sdlis4) {
+				if (sdlisten(sdnow, IP4SSL | IP6SSL)) {
 					c[sdnew].ssl = SSL_new(ssl_ctx);
 					SSL_set_fd(c[sdnew].ssl, sdnew);
 					c[sdnew].x_ssl = 1;
 				}
 
 				/* save ip and port */
-				if (sdnow == sdlis3 || sdnow == sdlis4) {
+				if (sdlisten(sdnow, IP6 | IP6SSL)) {
 					inet_ntop(AF_INET6, &ca6.sin6_addr,
 					    c[sdnew].ip, sizeof(c[sdnew].ip));
 					snprintf(c[sdnew].pt,
@@ -1190,7 +1367,7 @@ main(int argc, char *argv[])
 				c[sdnow].pfdh[i] = strdup(b);
 				c[sdnow].x_ful[i] = 1;
 				c[sdnow].x_chk[i] = 0;
-       				c[sdnow].x_sta = 0;
+				c[sdnow].x_sta = 0;
 				free(h);
 				free(b);
 				continue;
@@ -1232,7 +1409,7 @@ main(int argc, char *argv[])
 				c[sdnow].pfdh[i] = strdup(b);
 				c[sdnow].x_ful[i] = 1;
 				c[sdnow].x_chk[i] = 0;
-       				c[sdnow].x_sta = 0;
+				c[sdnow].x_sta = 0;
 				free(h);
 				free(b);
 				continue;
@@ -1256,7 +1433,7 @@ main(int argc, char *argv[])
 					c[sdnow].pfdh[i] = strdup(b);
 					c[sdnow].x_ful[i] = 1;
 					c[sdnow].x_chk[i] = 0;
-       					c[sdnow].x_sta = 0;
+					c[sdnow].x_sta = 0;
 					free(h);
 					free(b);
 					continue;
@@ -1306,7 +1483,7 @@ main(int argc, char *argv[])
 					c[sdnow].pfdh[i] = strdup(b);
 					c[sdnow].x_ful[i] = 1;
 					c[sdnow].x_chk[i] = 0;
-       					c[sdnow].x_sta = 0;
+					c[sdnow].x_sta = 0;
 					free(h);
 					free(b);
 				}
