@@ -458,7 +458,7 @@ sys_write_a(const int sfd, const char *buf, const int len)
 			}
 			if (errno == EAGAIN) {
 				sys_log(debug, "sys_write_a: EAGAIN: %d", sent);
-				return (sent);
+				return (sent > 0 ? sent : r);
 			}
 			if (errno != EPIPE)
 				syslog(LOG_ERR, "sys_write_a: %s",
@@ -542,7 +542,7 @@ sys_resetnonblock(const int fd)
 
 	flags &= ~O_NONBLOCK;
 	if (fcntl(fd, F_SETFL, flags) < 0)
-		return (-1); 
+		return (-1);
 
 	return (1);
 }
